// Globals
var numColumns = 16; // Number of tiles per column
var numRows = 16; // Number of tiles per row

var canvas = document.getElementById('game');
var context = canvas.getContext('2d'); 
var scoreSpan = document.getElementById('score');

var tileWidth = canvas.width / numColumns;
var tileHeight = canvas.height / numRows;

var tiles = new Array(numColumns * numRows);
var colors = ["Black", "Red", "Blue", "Green", "Yellow"];
var score = 0;
var remaining = 0;
var deletedTilesCount = 0;
var end = false;

//Runs the game for the first time
setUp();

function setUp() {
    score = 0;
    scoreSpan.innerHTML = score.toString();
	remaining = numColumns * numRows;
    for (var i = 0; i < tiles.length; i++) {
	    tiles[i] = Math.floor(1 + Math.random() * 4);
    }
    canvas.addEventListener("mouseup", processClick, false);
    draw();
}

//Loops through the entire array and draws each tile
function draw() {
	context.clearRect (0, 0, canvas.width, canvas.height);
	for (var row = 0 ; row < numRows; row++) {
		for (var column = 0; column < numColumns; column++) {
			drawTile(row, column);
		}
	}
}

//Draws individual tiles
function drawTile(row, column) {
	var pos = row*numColumns + column;
	var tileColor = tiles[pos];
	var color = colors[tileColor];
	var point = pairToPos(row, column);
    context.beginPath();
    context.rect(point.x, point.y, tileWidth, tileHeight);
    context.fillStyle = color;
    context.fill();
    context.lineWidth = 1;
    context.strokeStyle = 'black';
    context.stroke();
}

//Deletes the tiles that are connected to the clicked tile
//and are the same color as that tile (calls recursively)
function deleteTiles(row, column) {
    var pos = row * numColumns + column;
    var color = tiles[pos];

    if (color == 0) {
		return;
    }

    tiles[pos] = 0;
    remaining -= 1;
    deletedTilesCount++;

    if (row > 0 && color == tiles[pos - numColumns]) {
		deleteTiles(row - 1, column);
    }

    if (row < numRows - 1 && color == tiles[pos + numColumns]) {
		deleteTiles(row + 1, column);
    }

    if (column > 0 && color == tiles[pos - 1]) {
		deleteTiles(row, column - 1);
    }

    if (column < numColumns - 1 && color == tiles[pos + 1]) {
		deleteTiles(row, column + 1);
    }
}

//Checks to see if the clicked tile can be deleted
function checkDeletable(row, column) {
    var pos = row * numColumns + column;
    var color = tiles[pos];
    
    if (row > 0 && color == tiles[pos - numColumns]) {
		return true;
    }

    if (row < numRows - 1 && color == tiles[pos + numColumns]) {
		return true;
    }

    if (column > 0 && color == tiles[pos - 1]) {
		return true;
    }

    if (column < numColumns - 1 && color == tiles[pos + 1]) {
		return true;
    }

    return false;
}

//Takes a click and runs the game accordingly
function processClick(event) {
    var xCoord = new Number();
    var yCoord = new Number();

    if (event.x !== undefined && event.y !== undefined) {
		xCoord = event.x;
		yCoord = event.y;
    }
    else {
	xCoord = event.clientX + document.body.scrollLeft +
	    document.documentElement.scrollLeft;
	yCoord = event.clientY + document.body.scrollTop +
	    document.documentElement.scrollTop;
    }

    xCoord -= canvas.offsetLeft;
    yCoord -= canvas.offsetTop;

    var pos = posToPair(xCoord, yCoord);
    
    deletedTilesCount = 0;
    if (checkDeletable(pos.row, pos.column)) {
	    deleteTiles(pos.row, pos.column);
    }
    if (deletedTilesCount > 0) {
        score += deletedTilesCount*(deletedTilesCount - 1);
        scoreSpan.innerHTML = score.toString();
    }
    moveTiles();
    draw();
    checkGameOver();
}

//Repositions the tiles
function moveTiles() {
	//First count the number of deleted blocks in a column
    for (var col = 0; col < numColumns; col++) {
		var blackCount = 0;
		//Then move each colored block down by that many
		for (var row = numRows - 1; row >= 0; row--) {
		    var pos = row * numColumns + col;
		    if (tiles[pos] == 0) {
				blackCount++;
		    } else if (blackCount > 0) {
				tiles[pos + blackCount * numColumns] = tiles[pos];
				tiles[pos] = 0;
		    }
		}
    }

    var blackCount = 0;
    //Same logic as above except by checking number of deleted columns
    for (var col = 0; col < numColumns; col++) {
		if (tiles[(numRows - 1) * numColumns + col] == 0) {
		    blackCount++;
		} else if (blackCount > 0) {
		    for (var row = 0; row < numRows; row++) {
				var pos = row * numColumns + col;
				tiles[pos - blackCount] = tiles[pos];
				tiles[pos] = 0;
		    }
		}
    }
}

//Checks the entire grid for adjacent colors
function checkAdjacent() {
	for(var col = 0; col < numColumns; col++) {	
		for(var row = numRows - 1; row >= 0; row--) {
			var color = tiles[row * numColumns + col];
			if(color === 0) {
				break;
			} else {
				if(row - 1 >= 0 && tiles[(row - 1) * numColumns + col] == color) {
					return false;
				} else if(col + 1 < numColumns && tiles[row * numColumns + col + 1] == color) {
					return false;
				}
			}
		}
	}
	return true;
}

//After the player clicks and the events are processed
//check to see if the game is over
function checkGameOver() {
	if (remaining === 0 || checkAdjacent()) {
		context.clearRect(0, 0, canvas.width, canvas.height);
		context.fillStyle = 'black';
		context.font="20px Georgia";
		context.textAlign = 'center';
		context.fillText("Game Over", canvas.width / 2, canvas.height / 2);
		context.font="10px Georgia";
		context.textAlign = 'center';
		context.fillText("Click to restart", canvas.width / 2, canvas.height / 2 + 25)
		
		canvas.removeEventListener("mouseup", processClick, false);
		canvas.addEventListener("mouseup", restart, false);
		end = true;
	}
}

//Restart the game using the canvas
function restart(event) {
	canvas.removeEventListener("mouseup", restart, false);
	setUp();
}

//Restart the game using the button
function restartButton() {
	if(end) {
		canvas.removeEventListener("mouseup", processClick, false);
		canvas.removeEventListener("mouseup", restart, false);
	}
	setUp();
}

//Converts an row and a column to the top left corner of the tile
//for drawing the tile
function pairToPos(row, column) {
    var Point = {
	    x: column * tileWidth,
	    y: row * tileHeight
    };
    
    return Point;
}

//Converts an x, y pair to a position in the grid
function posToPair(x, y) {
    var Pos = {
	    row: Math.floor(y / tileHeight),
	    column: Math.floor(x / tileWidth)
    };

    return Pos;
}
